@@ -21,6 +21,11 @@
 #ifndef __BTRFS_CHECK_MODE_ORIGINAL_H__
 #define __BTRFS_CHECK_MODE_ORIGINAL_H__
 
+#include "kerncompat.h"
+#include "kernel-lib/rbtree.h"
+#include "kernel-lib/list.h"
+#include "kernel-shared/uapi/btrfs_tree.h"
+#include "common/extent-cache.h"
 #include "common/rbtree-utils.h"
 
 struct extent_backref {
@@ -258,7 +263,10 @@ struct root_record {
 	struct cache_extent cache;
 	unsigned int found_root_item:1;
 	u64 objectid;
+	/* The found number of refs in tree root. */
 	u32 found_ref;
+	/* The expected number of refs in root item. */
+	u32 expected_ref;
 };
 
 struct ptr_node {
