@@ -161,10 +161,10 @@ static int load_chunk_info(int fd, struct array *chunkinfos)
 	sk->tree_id = BTRFS_CHUNK_TREE_OBJECTID;
 
 	sk->min_objectid = 0;
-	sk->max_objectid = (u64)-1;
-	sk->max_type = 0;
 	sk->min_type = (u8)-1;
 	sk->min_offset = 0;
+	sk->max_objectid = (u64)-1;
+	sk->max_type = 0;
 	sk->max_offset = (u64)-1;
 	sk->min_transid = 0;
 	sk->max_transid = (u64)-1;
@@ -622,7 +622,7 @@ static int print_filesystem_usage_overall(int fd, const struct array *chunkinfos
 	 */
 	unreliable_allocated = (raid56 && chunkinfos->length == 0);
 	if (unreliable_allocated) {
-		warning("radid56 found, we cannots compute some values, run as root if needed");
+		warning("raid56 found, we cannot compute some values, run as root if needed");
 		ret = 1;
 		goto exit;
 	}
@@ -820,8 +820,14 @@ static int load_device_info(int fd, struct array *devinfos)
 			strcpy(info->path, "missing");
 		} else {
 			strcpy(info->path, (char *)dev_info.path);
-			info->device_size =
-				device_get_partition_size((const char *)dev_info.path);
+			ret = device_get_partition_size((const char *)dev_info.path,
+							&info->device_size);
+			if (ret < 0) {
+				errno = -ret;
+				warning("failed to get device size for %s: %m",
+					dev_info.path);
+				info->device_size = 0;
+			}
 		}
 		info->size = dev_info.total_bytes;
 		ndevs++;
@@ -959,7 +965,6 @@ static void _cmd_filesystem_usage_tabular(unsigned unit_mode,
 		int k;
 		const char *p;
 		const struct device_info *devinfo = devinfos->data[i];
-
 		u64  total_allocated = 0, unused;
 
 		p = strrchr(devinfo->path, '/');
@@ -1000,7 +1005,6 @@ static void _cmd_filesystem_usage_tabular(unsigned unit_mode,
 			col++;
 		}
 
-		unused = device_get_partition_size(devinfo->path) - total_allocated;
 		unused = devinfo->size - total_allocated;
 
 		table_printf(matrix, unallocated_col, vhdr_skip + i, ">%s",
