@@ -33,8 +33,8 @@
 #include <stdarg.h>
 #include <limits.h>
 #include <dirent.h>
-#include <errno.h>
 #include <getopt.h>
+#include <errno.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -45,13 +45,17 @@
 #include "kernel-lib/sizes.h"
 #include "kernel-shared/volumes.h"
 #include "common/defs.h"
+#include "common/compat.h"
 #include "common/messages.h"
 #include "common/utils.h"
 #include "common/open-utils.h"
 #include "common/units.h"
+#include "common/device-utils.h"
+#include "common/sysfs-utils.h"
+#include "common/string-table.h"
+#include "common/string-utils.h"
 #include "common/help.h"
 #include "cmds/commands.h"
-#include "ioctl.h"
 
 static unsigned unit_mode = UNITS_DEFAULT;
 
@@ -94,6 +98,8 @@ struct scrub_progress {
 	pthread_mutex_t progress_mutex;
 	int ioprio_class;
 	int ioprio_classdata;
+	u64 old_limit;
+	u64 limit;
 };
 
 struct scrub_file_record {
@@ -144,7 +150,7 @@ static void print_scrub_full(struct btrfs_scrub_progress *sp)
 } while (0)
 
 static void print_scrub_summary(struct btrfs_scrub_progress *p, struct scrub_stats *s,
-		u64 bytes_total)
+				u64 bytes_total, u64 limit)
 {
 	u64 err_cnt;
 	u64 err_cnt2;
@@ -154,7 +160,14 @@ static void print_scrub_summary(struct btrfs_scrub_progress *p, struct scrub_sta
 	time_t sec_eta;
 
 	bytes_scrubbed = p->data_bytes_scrubbed + p->tree_bytes_scrubbed;
-	if (s->duration > 0)
+	/*
+	 * If duration is zero seconds (rounded down), then the Rate metric
+	 * should still reflect the amount of bytes that have been processed
+	 * in under a second.
+	 */
+	if (s->duration == 0)
+		bytes_per_sec = bytes_scrubbed;
+	else
 		bytes_per_sec = bytes_scrubbed / s->duration;
 	if (bytes_per_sec > 0)
 		sec_left = (bytes_total - bytes_scrubbed) / bytes_per_sec;
@@ -192,16 +205,34 @@ static void print_scrub_summary(struct btrfs_scrub_progress *p, struct scrub_sta
 		pr_verbose(LOG_DEFAULT, "Total to scrub:   %s\n",
 			pretty_size_mode(bytes_total, unit_mode));
 	}
+
 	/*
 	 * Rate and size units are disproportionate so they are affected only
-	 * by --raw, otherwise it's human readable
+	 * by --raw, otherwise it's human readable (respecting the SI or IEC mode).
 	 */
 	if (unit_mode == UNITS_RAW) {
-		pr_verbose(LOG_DEFAULT, "Rate:             %s/s\n",
+		pr_verbose(LOG_DEFAULT, "Rate:             %s/s",
 			pretty_size_mode(bytes_per_sec, UNITS_RAW));
+		if (limit > 1)
+			pr_verbose(LOG_DEFAULT, " (limit %s/s)",
+				   pretty_size_mode(limit, UNITS_RAW));
+		else if (limit == 1)
+			pr_verbose(LOG_DEFAULT, " (some device limits set)");
+		pr_verbose(LOG_DEFAULT, "\n");
 	} else {
-		pr_verbose(LOG_DEFAULT, "Rate:             %s/s\n",
-			pretty_size(bytes_per_sec));
+		unsigned int mode = UNITS_HUMAN_DECIMAL;
+
+		if (unit_mode & UNITS_BINARY)
+			mode = UNITS_HUMAN_BINARY;
+
+		pr_verbose(LOG_DEFAULT, "Rate:             %s/s",
+			pretty_size_mode(bytes_per_sec, mode));
+		if (limit > 1)
+			pr_verbose(LOG_DEFAULT, " (limit %s/s)",
+				   pretty_size_mode(limit, mode));
+		else if (limit == 1)
+			pr_verbose(LOG_DEFAULT, " (some device limits set)");
+		pr_verbose(LOG_DEFAULT, "\n");
 	}
 
 	pr_verbose(LOG_DEFAULT, "Error summary:   ");
@@ -315,7 +346,8 @@ static void _print_scrub_ss(struct scrub_stats *ss)
 
 static void print_scrub_dev(struct btrfs_ioctl_dev_info_args *di,
 				struct btrfs_scrub_progress *p, int raw,
-				const char *append, struct scrub_stats *ss)
+				const char *append, struct scrub_stats *ss,
+				u64 limit)
 {
 	pr_verbose(LOG_DEFAULT, "\nScrub device %s (id %llu) %s\n", di->path, di->devid,
 	       append ? append : "");
@@ -323,21 +355,48 @@ static void print_scrub_dev(struct btrfs_ioctl_dev_info_args *di,
 	_print_scrub_ss(ss);
 
 	if (p) {
-		if (raw)
+		if (raw) {
 			print_scrub_full(p);
-		else
-			print_scrub_summary(p, ss, di->bytes_used);
+		} else if (ss->finished) {
+			/*
+			 * For finished scrub, we can use the total scrubbed
+			 * bytes to report "Total to scrub", which is more
+			 * accurate (e.g. mostly empty block groups).
+			 */
+			print_scrub_summary(p, ss, p->data_bytes_scrubbed +
+					    p->tree_bytes_scrubbed, limit);
+		} else {
+			/*
+			 * For any canceled/interrupted/running scrub, we're
+			 * not sure how many bytes we're really going to scrub,
+			 * thus we use device's used bytes instead.
+			 */
+			print_scrub_summary(p, ss, di->bytes_used, limit);
+		}
 	}
 }
 
-static void print_fs_stat(struct scrub_fs_stat *fs_stat, int raw, u64 bytes_total)
+/*
+ * Print summary stats for the whole filesystem. If there's only one device
+ * print the limit if set, otherwise a special value to print a note that
+ * limits are set.
+ */
+static void print_fs_stat(struct scrub_fs_stat *fs_stat, int raw, u64 bytes_total,
+			  u64 nr_devices, u64 limit)
 {
 	_print_scrub_ss(&fs_stat->s);
 
-	if (raw)
+	if (raw) {
 		print_scrub_full(&fs_stat->p);
-	else
-		print_scrub_summary(&fs_stat->p, &fs_stat->s, bytes_total);
+	} else {
+		/*
+		 * Limit for the whole filesystem stats does not make sense,
+		 * but if there's any device with a limit then print it.
+		 */
+		if (nr_devices != 1 && limit)
+			limit = 1;
+		print_scrub_summary(&fs_stat->p, &fs_stat->s, bytes_total, limit);
+	}
 }
 
 static void free_history(struct scrub_file_record **last_scrubs)
@@ -602,7 +661,7 @@ again:
 			memset(p[curr], 0, sizeof(**p));
 			p[curr + 1] = NULL;
 			++state;
-			/* fall through */
+			fallthrough;
 		case 2: /* start of line, skip space */
 			while (isspace(l[i]) && i < avail) {
 				if (l[i] == '\n')
@@ -613,7 +672,7 @@ again:
 			    (!eof && !memchr(l + i, '\n', avail - i)))
 				goto again;
 			++state;
-			/* fall through */
+			fallthrough;
 		case 3: /* read fsid */
 			if (i == avail)
 				continue;
@@ -629,7 +688,7 @@ again:
 				_SCRUB_INVALID;
 			i += j + 1;
 			++state;
-			/* fall through */
+			fallthrough;
 		case 4: /* read dev id */
 			for (j = 0; isdigit(l[i + j]) && i+j < avail; ++j)
 				;
@@ -638,7 +697,7 @@ again:
 			p[curr]->devid = atoll(&l[i]);
 			i += j + 1;
 			++state;
-			/* fall through */
+			fallthrough;
 		case 5: /* read key/value pair */
 			ret = 0;
 			_SCRUB_KVREAD(ret, &i, data_extents_scrubbed, avail, l,
@@ -682,7 +741,7 @@ again:
 			if (ret != 1)
 				_SCRUB_INVALID;
 			++state;
-			/* fall through */
+			fallthrough;
 		case 6: /* after number */
 			if (l[i] == '|')
 				state = 5;
@@ -1146,6 +1205,31 @@ static int is_scrub_running_in_kernel(int fd,
 	return 0;
 }
 
+static u64 read_scrub_device_limit(int fd, u64 devid)
+{
+	char path[PATH_MAX] = { 0 };
+	u64 limit;
+	int ret;
+
+	/* /sys/fs/btrfs/FSID/devinfo/1/scrub_speed_max */
+	snprintf(path, sizeof(path), "devinfo/%llu/scrub_speed_max", devid);
+	ret = sysfs_read_fsid_file_u64(fd, path, &limit);
+	if (ret < 0)
+		limit = 0;
+	return limit;
+}
+
+static u64 write_scrub_device_limit(int fd, u64 devid, u64 limit)
+{
+	char path[PATH_MAX] = { 0 };
+	int ret;
+
+	/* /sys/fs/btrfs/FSID/devinfo/1/scrub_speed_max */
+	snprintf(path, sizeof(path), "devinfo/%llu/scrub_speed_max", devid);
+	ret = sysfs_write_fsid_file_u64(fd, path, limit);
+	return ret;
+}
+
 static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 		       bool resume)
 {
@@ -1154,7 +1238,6 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 	int fdres = -1;
 	int ret;
 	pid_t pid;
-	int c;
 	int i;
 	int err = 0;
 	int e_uncorrectable = 0;
@@ -1164,14 +1247,12 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 	bool do_background = true;
 	bool do_wait = false;
 	bool do_print = false;
-	int do_quiet = !bconf.verbose; /*Read the global quiet option if set*/
 	bool do_record = true;
 	bool readonly = false;
 	bool do_stats_per_dev = false;
 	int ioprio_class = IOPRIO_CLASS_IDLE;
 	int ioprio_classdata = 0;
 	int n_start = 0;
-	int n_skip = 0;
 	int n_resume = 0;
 	struct btrfs_ioctl_fs_info_args fi_args;
 	struct btrfs_ioctl_dev_info_args *di_args = NULL;
@@ -1185,18 +1266,28 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 	pthread_t t_prog;
 	struct scrub_file_record **past_scrubs = NULL;
 	struct scrub_file_record *last_scrub = NULL;
-	char *datafile = strdup(SCRUB_DATA_FILE);
+	char datafile[] = SCRUB_DATA_FILE;
 	char fsid[BTRFS_UUID_UNPARSED_SIZE];
 	char sock_path[PATH_MAX] = "";
 	struct scrub_progress_cycle spc;
 	pthread_mutex_t spc_write_mutex = PTHREAD_MUTEX_INITIALIZER;
 	void *terr;
+	u64 throughput_limit = 0;
 	u64 devid;
-	DIR *dirstream = NULL;
 	bool force = false;
 	bool nothing_to_resume = false;
 
-	while ((c = getopt(argc, argv, "BdqrRc:n:f")) != -1) {
+	while (1) {
+		int c;
+		enum { GETOPT_VAL_LIMIT = GETOPT_VAL_FIRST };
+		static const struct option long_options[] = {
+			{"limit", required_argument, NULL, GETOPT_VAL_LIMIT},
+			{ NULL, 0, NULL, 0 }
+		};
+
+		c = getopt_long(argc, argv, "BdqrRc:n:f", long_options, NULL);
+		if (c < 0)
+			break;
 		switch (c) {
 		case 'B':
 			do_background = false;
@@ -1208,7 +1299,6 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 			break;
 		case 'q':
 			bconf_be_quiet();
-			do_quiet = !bconf.verbose;
 			break;
 		case 'r':
 			readonly = true;
@@ -1225,6 +1315,9 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 		case 'f':
 			force = true;
 			break;
+		case GETOPT_VAL_LIMIT:
+			throughput_limit = arg_strtou64_with_suffix(optarg);
+			break;
 		default:
 			usage_unknown_option(cmd, argv);
 		}
@@ -1236,33 +1329,30 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 		return 1;
 
 	spc.progress = NULL;
-	if (do_quiet && do_print)
+	if (bconf.verbose == BTRFS_BCONF_QUIET && do_print)
 		do_print = false;
 
 	if (mkdir_p(datafile)) {
-		warning_on(!do_quiet,
-    "cannot create scrub data file, mkdir %s failed: %m. Status recording disabled",
+		warning("cannot create scrub data file, mkdir %s failed: %m, status recording disabled",
 			datafile);
 		do_record = false;
 	}
-	free(datafile);
 
 	path = argv[optind];
 
-	fdmnt = open_path_or_dev_mnt(path, &dirstream, !do_quiet);
+	fdmnt = btrfs_open_mnt(path);
 	if (fdmnt < 0)
 		return 1;
 
 	ret = get_fs_info(path, &fi_args, &di_args);
 	if (ret) {
 		errno = -ret;
-		error_on(!do_quiet,
-			"getting dev info for scrub failed: %m");
+		error("getting dev info for scrub failed: %m");
 		err = 1;
 		goto out;
 	}
 	if (!fi_args.num_devices) {
-		error_on(!do_quiet, "no devices found");
+		error("no devices found");
 		err = 1;
 		goto out;
 	}
@@ -1271,12 +1361,12 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 	fdres = scrub_open_file_r(SCRUB_DATA_FILE, fsid);
 	if (fdres < 0 && fdres != -ENOENT) {
 		errno = -fdres;
-		warning_on(!do_quiet, "failed to open status file: %m");
+		warning("failed to open status file: %m");
 	} else if (fdres >= 0) {
-		past_scrubs = scrub_read_file(fdres, !do_quiet);
+		past_scrubs = scrub_read_file(fdres, 1);
 		if (IS_ERR(past_scrubs)) {
 			errno = -PTR_ERR(past_scrubs);
-			warning_on(!do_quiet, "failed to read status file: %m");
+			warning("failed to read status file: %m");
 		}
 		close(fdres);
 	}
@@ -1300,8 +1390,7 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 	 * single devices, there is no reason to prevent this.
 	 */
 	if (!force && is_scrub_running_on_fs(&fi_args, di_args, past_scrubs)) {
-		error_on(!do_quiet,
-			"Scrub is already running.\n"
+		error(  "Scrub is already running.\n"
 			"To cancel use 'btrfs scrub cancel %s'.\n"
 			"To see the status use 'btrfs scrub status [-d] %s'",
 			path, path);
@@ -1314,17 +1403,24 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 	spc.progress = calloc(fi_args.num_devices * 2, sizeof(*spc.progress));
 
 	if (!t_devs || !sp || !spc.progress) {
-		error_on(!do_quiet, "scrub failed: %m");
+		error("scrub failed: %m");
 		err = 1;
 		goto out;
 	}
 
 	for (i = 0; i < fi_args.num_devices; ++i) {
 		devid = di_args[i].devid;
+		sp[i].old_limit = read_scrub_device_limit(fdmnt, devid);
+		ret = write_scrub_device_limit(fdmnt, devid, throughput_limit);
+		if (ret < 0) {
+			errno = -ret;
+			warning("failed to set scrub throughput limit on devid %llu: %m",
+				devid);
+		}
 		ret = pthread_mutex_init(&sp[i].progress_mutex, NULL);
 		if (ret) {
 			errno = ret;
-			error_on(!do_quiet, "pthread_mutex_init failed: %m");
+			error("pthread_mutex_init failed: %m");
 			err = 1;
 			goto out;
 		}
@@ -1337,7 +1433,6 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 			sp[i].scrub_args.start = last_scrub->p.last_physical;
 			sp[i].resumed = last_scrub;
 		} else if (resume) {
-			++n_skip;
 			sp[i].skip = 1;
 			sp[i].resumed = last_scrub;
 			continue;
@@ -1351,6 +1446,7 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 		sp[i].scrub_args.flags = readonly ? BTRFS_SCRUB_READONLY : 0;
 		sp[i].ioprio_class = ioprio_class;
 		sp[i].ioprio_classdata = ioprio_classdata;
+		sp[i].limit = read_scrub_device_limit(fdmnt, devid);
 	}
 
 	if (!n_start && !n_resume) {
@@ -1367,7 +1463,7 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 					sock_path, sizeof(sock_path));
 		/* ignore EOVERFLOW, try using a shorter path for the socket */
 		addr.sun_path[sizeof(addr.sun_path) - 1] = '\0';
-		strncpy(addr.sun_path, sock_path, sizeof(addr.sun_path) - 1);
+		strncpy_null(addr.sun_path, sock_path, sizeof(addr.sun_path));
 		ret = bind(prg_fd, (struct sockaddr *)&addr, sizeof(addr));
 		if (ret != -1 || errno != EADDRINUSE)
 			break;
@@ -1392,8 +1488,7 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 	if (ret != -1)
 		ret = listen(prg_fd, 100);
 	if (ret == -1) {
-		warning_on(!do_quiet,
-   "failed to open the progress status socket at %s: %m. Progress cannot be queried",
+		warning("failed to open the progress status socket at %s: %m, progress cannot be queried",
 			sock_path[0] ? sock_path :
 			SCRUB_PROGRESS_SOCKET_PATH);
 		if (prg_fd != -1) {
@@ -1410,8 +1505,7 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 					   fi_args.num_devices);
 		if (ret) {
 			errno = -ret;
-			warning_on(!do_quiet,
-   "failed to write the progress status file: %m. Status recording disabled");
+			warning("failed to write the progress status file: %m, status recording disabled");
 			do_record = false;
 		}
 	}
@@ -1419,7 +1513,7 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 	if (do_background) {
 		pid = fork();
 		if (pid == -1) {
-			error_on(!do_quiet, "cannot scrub, fork failed: %m");
+			error("cannot scrub, fork failed: %m");
 			err = 1;
 			goto out;
 		}
@@ -1437,14 +1531,13 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 			}
 			ret = wait(&stat);
 			if (ret != pid) {
-				error_on(!do_quiet, "wait failed (ret=%d): %m",
-					ret);
+				error("wait failed (ret=%d): %m", ret);
 				err = 1;
 				goto out;
 			}
 			if (!WIFEXITED(stat) || WEXITSTATUS(stat)) {
-				error_on(!do_quiet, "scrub process failed");
 				err = WIFEXITED(stat) ? WEXITSTATUS(stat) : -1;
+				error("scrub process failed with error %d", err);
 				goto out;
 			}
 			err = 0;
@@ -1465,6 +1558,12 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 		devid = di_args[i].devid;
 		gettimeofday(&tv, NULL);
 		sp[i].stats.t_start = tv.tv_sec;
+		pr_verbose(LOG_DEFAULT, "Starting scrub on devid %llu", devid);
+		if (sp[i].limit > 0)
+			pr_verbose(LOG_DEFAULT, " (limit %s/s)\n", pretty_size(sp[i].limit));
+		else
+			pr_verbose(LOG_DEFAULT, "\n");
+
 		ret = pthread_create(&t_devs[i], NULL,
 					scrub_one_dev, &sp[i]);
 		if (ret) {
@@ -1497,6 +1596,14 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 
 	err = 0;
 	for (i = 0; i < fi_args.num_devices; ++i) {
+		/* Revert to the older scrub limit. */
+		ret = write_scrub_device_limit(fdmnt, di_args[i].devid, sp[i].old_limit);
+		if (ret < 0) {
+			errno = -ret;
+			warning("failed to reset scrub throughput limit on devid %llu: %m",
+				di_args[i].devid);
+		}
+
 		if (sp[i].skip)
 			continue;
 		devid = di_args[i].devid;
@@ -1542,28 +1649,38 @@ static int scrub_start(const struct cmd_struct *cmd, int argc, char **argv,
 
 	if (do_print) {
 		const char *append = "done";
-		u64 total_bytes_used = 0;
+		u64 total_bytes_scrubbed = 0;
+		u64 limit = 0;
 
 		if (!do_stats_per_dev)
 			init_fs_stat(&fs_stat);
 		for (i = 0; i < fi_args.num_devices; ++i) {
+			struct btrfs_scrub_progress *cur_progress =
+						&sp[i].scrub_args.progress;
+
+			/* On a multi-device filesystem, keep the lowest limit only. */
+			if (!limit || (sp[i].limit && sp[i].limit < limit))
+				limit = sp[i].limit;
+
 			if (do_stats_per_dev) {
 				print_scrub_dev(&di_args[i],
-						&sp[i].scrub_args.progress,
+						cur_progress,
 						print_raw,
 						sp[i].ret ? "canceled" : "done",
-						&sp[i].stats);
+						&sp[i].stats,
+						sp[i].limit);
 			} else {
 				if (sp[i].ret)
 					append = "canceled";
-				add_to_fs_stat(&sp[i].scrub_args.progress,
-						&sp[i].stats, &fs_stat);
+				add_to_fs_stat(cur_progress, &sp[i].stats, &fs_stat);
 			}
-			total_bytes_used += di_args[i].bytes_used;
+			total_bytes_scrubbed += cur_progress->data_bytes_scrubbed +
+						cur_progress->tree_bytes_scrubbed;
 		}
 		if (!do_stats_per_dev) {
 			pr_verbose(LOG_DEFAULT, "scrub %s for %s\n", append, fsid);
-			print_fs_stat(&fs_stat, print_raw, total_bytes_used);
+			print_fs_stat(&fs_stat, print_raw, total_bytes_scrubbed,
+				      fi_args.num_devices, limit);
 		}
 	}
 
@@ -1605,37 +1722,37 @@ out:
 		if (sock_path[0])
 			unlink(sock_path);
 	}
-	close_file_or_dir(fdmnt, dirstream);
+	close(fdmnt);
 
 	if (err)
 		return 1;
 	if (nothing_to_resume)
 		return 2;
 	if (e_uncorrectable) {
-		error_on(!do_quiet, "there are uncorrectable errors");
+		error("there are %d uncorrectable errors", e_uncorrectable);
 		return 3;
 	}
 	if (e_correctable)
-		warning_on(!do_quiet,
-			"errors detected during scrubbing, corrected");
+		warning("errors detected during scrubbing, %d corrected", e_correctable);
 
 	return 0;
 }
 
 static const char * const cmd_scrub_start_usage[] = {
-	"btrfs scrub start [-BdqrRf] [-c ioprio_class -n ioprio_classdata] <path>|<device>",
-	"Start a new scrub. If a scrub is already running, the new one fails.",
+	"btrfs scrub start [options] <path>|<device>",
+	"Start a new scrub on the filesystem or a device (can be started only once)",
 	"",
-	"-B     do not background",
-	"-d     stats per device (-B only)",
-	"-q     be quiet",
-	"-r     read only mode",
-	"-R     raw print mode, print full data instead of summary",
-	"-c     set ioprio class (see ionice(1) manpage)",
-	"-n     set ioprio classdata (see ionice(1) manpage)",
-	"-f     force starting new scrub even if a scrub is already running",
-	"       this is useful when scrub stats record file is damaged",
-	"-q     deprecated, alias for global -q option",
+	OPTLINE("-B", "do not background"),
+	OPTLINE("-d", "stats per device (-B only)"),
+	OPTLINE("-r", "read only mode (no repairs done)"),
+	OPTLINE("-R", "raw print mode, print full data instead of summary"),
+	OPTLINE("--limit SIZE", "set the throughput limit for each device (0 for unlimited), restored afterwards"),
+	OPTLINE("-f", "force starting new scrub even if a scrub is already running this is useful when scrub stats record file is damaged"),
+	OPTLINE("-q", "deprecated, alias for global -q option"),
+	"",
+	"Priority (requires IO scheduler support, not supported by mq-deadline):",
+	OPTLINE("-c CLASS ", "set ioprio class (see ionice(1) manpage), "),
+	OPTLINE("-n CDATA", "set ioprio classdata (see ionice(1) manpage)"),
 	HELPINFO_INSERT_GLOBALS,
 	HELPINFO_INSERT_QUIET,
 	NULL
@@ -1660,7 +1777,6 @@ static int cmd_scrub_cancel(const struct cmd_struct *cmd, int argc, char **argv)
 	char *path;
 	int ret;
 	int fdmnt = -1;
-	DIR *dirstream = NULL;
 
 	clean_args_no_options(cmd, argc, argv);
 
@@ -1669,7 +1785,7 @@ static int cmd_scrub_cancel(const struct cmd_struct *cmd, int argc, char **argv)
 
 	path = argv[optind];
 
-	fdmnt = open_path_or_dev_mnt(path, &dirstream, 1);
+	fdmnt = btrfs_open_mnt(path);
 	if (fdmnt < 0) {
 		ret = 1;
 		goto out;
@@ -1691,7 +1807,7 @@ static int cmd_scrub_cancel(const struct cmd_struct *cmd, int argc, char **argv)
 	pr_verbose(LOG_DEFAULT, "scrub cancelled\n");
 
 out:
-	close_file_or_dir(fdmnt, dirstream);
+	close(fdmnt);
 	return ret;
 }
 static DEFINE_SIMPLE_COMMAND(scrub_cancel, "cancel");
@@ -1700,13 +1816,15 @@ static const char * const cmd_scrub_resume_usage[] = {
 	"btrfs scrub resume [-BdqrR] [-c ioprio_class -n ioprio_classdata] <path>|<device>",
 	"Resume previously canceled or interrupted scrub",
 	"",
-	"-B     do not background",
-	"-d     stats per device (-B only)",
-	"-r     read only mode",
-	"-R     raw print mode, print full data instead of summary",
-	"-c     set ioprio class (see ionice(1) manpage)",
-	"-n     set ioprio classdata (see ionice(1) manpage)",
-	"-q     deprecated, alias for global -q option",
+	OPTLINE("-B", "do not background"),
+	OPTLINE("-d", "stats per device (-B only)"),
+	OPTLINE("-r", "read only mode"),
+	OPTLINE("-R", "raw print mode, print full data instead of summary"),
+	OPTLINE("-q", "deprecated, alias for global -q option"),
+	"",
+	"Priority (requires IO scheduler support, not supported by mq-deadline):",
+	OPTLINE("-c CLASS", "set ioprio class (see ionice(1) manpage)"),
+	OPTLINE("-n CDATA", "set ioprio classdata (see ionice(1) manpage)"),
 	HELPINFO_INSERT_GLOBALS,
 	HELPINFO_INSERT_QUIET,
 	NULL
@@ -1722,8 +1840,8 @@ static const char * const cmd_scrub_status_usage[] = {
 	"btrfs scrub status [-dR] <path>|<device>",
 	"Show status of running or finished scrub",
 	"",
-	"-d                 stats per device",
-	"-R                 print raw stats",
+	OPTLINE("-d", "stats per device"),
+	OPTLINE("-R", "print raw stats"),
 	HELPINFO_UNITS_LONG,
 	NULL
 };
@@ -1750,7 +1868,6 @@ static int cmd_scrub_status(const struct cmd_struct *cmd, int argc, char **argv)
 	char fsid[BTRFS_UUID_UNPARSED_SIZE];
 	int fdres = -1;
 	int err = 0;
-	DIR *dirstream = NULL;
 
 	unit_mode = get_unit_mode_from_arg(&argc, argv, 0);
 
@@ -1773,7 +1890,7 @@ static int cmd_scrub_status(const struct cmd_struct *cmd, int argc, char **argv)
 
 	path = argv[optind];
 
-	fdmnt = open_path_or_dev_mnt(path, &dirstream, 1);
+	fdmnt = btrfs_open_mnt(path);
 	if (fdmnt < 0)
 		return 1;
 
@@ -1834,26 +1951,35 @@ static int cmd_scrub_status(const struct cmd_struct *cmd, int argc, char **argv)
 
 	if (do_stats_per_dev) {
 		for (i = 0; i < fi_args.num_devices; ++i) {
+			u64 limit;
+
+			limit = read_scrub_device_limit(fdmnt, di_args[i].devid);
 			last_scrub = last_dev_scrub(past_scrubs,
 							di_args[i].devid);
 			if (!last_scrub) {
 				print_scrub_dev(&di_args[i], NULL, print_raw,
-						NULL, NULL);
+						NULL, NULL, limit);
 				continue;
 			}
 			last_scrub->stats.in_progress = in_progress;
 			print_scrub_dev(&di_args[i], &last_scrub->p, print_raw,
 					last_scrub->stats.finished ?
 							"history" : "status",
-					&last_scrub->stats);
+					&last_scrub->stats, limit);
 		}
 	} else {
 		u64 total_bytes_used = 0;
 		struct btrfs_ioctl_space_info *sp = si_args->spaces;
+		u64 limit = 0;
 
 		init_fs_stat(&fs_stat);
 		fs_stat.s.in_progress = in_progress;
 		for (i = 0; i < fi_args.num_devices; ++i) {
+			/* On a multi-device filesystem, keep the lowest limit only. */
+			u64 this_limit = read_scrub_device_limit(fdmnt, di_args[i].devid);
+			if (!limit || (this_limit && this_limit < limit))
+				limit = this_limit;
+
 			last_scrub = last_dev_scrub(past_scrubs,
 							di_args[i].devid);
 			if (!last_scrub)
@@ -1868,7 +1994,8 @@ static int cmd_scrub_status(const struct cmd_struct *cmd, int argc, char **argv)
 			/* This is still slightly off for RAID56 */
 			total_bytes_used += sp->used_bytes * factor;
 		}
-		print_fs_stat(&fs_stat, print_raw, total_bytes_used);
+		print_fs_stat(&fs_stat, print_raw, total_bytes_used,
+			      fi_args.num_devices, limit);
 	}
 
 out:
@@ -1877,12 +2004,218 @@ out:
 	free(si_args);
 	if (fdres > -1)
 		close(fdres);
-	close_file_or_dir(fdmnt, dirstream);
+	close(fdmnt);
 
 	return !!err;
 }
 static DEFINE_SIMPLE_COMMAND(scrub_status, "status");
 
+static const char * const cmd_scrub_limit_usage[] = {
+	"btrfs scrub limit [options] <path>",
+	"Show or set scrub limits on devices of the given filesystem.",
+	"",
+	OPTLINE("-a|--all", "apply the limit to all devices"),
+	OPTLINE("-d|--devid DEVID", "select the device by DEVID to apply the limit"),
+	OPTLINE("-l|--limit SIZE", "set the limit of the device to SIZE (size units with suffix), or 0 to reset to unlimited"),
+	HELPINFO_UNITS_LONG,
+	NULL
+};
+
+static int cmd_scrub_limit(const struct cmd_struct *cmd, int argc, char **argv)
+{
+	struct btrfs_ioctl_fs_info_args fi_args = { 0 };
+	char fsid[BTRFS_UUID_UNPARSED_SIZE];
+	struct string_table *table = NULL;
+	int ret;
+	int fd = -1;
+	int cols, idx;
+	u64 opt_devid = 0;
+	bool devid_set = false;
+	u64 opt_limit = 0;
+	bool limit_set = false;
+	bool all_set = false;
+
+	unit_mode = get_unit_mode_from_arg(&argc, argv, 0);
+
+	optind = 0;
+	while (1) {
+		int c;
+		static const struct option long_options[] = {
+			{ "all", no_argument, NULL, 'a' },
+			{ "devid", required_argument, NULL, 'd' },
+			{ "limit", required_argument, NULL, 'l' },
+			{ NULL, 0, NULL, 0 }
+		};
+
+		c = getopt_long(argc, argv, "ad:l:", long_options, NULL);
+		if (c < 0)
+			break;
+
+		switch (c) {
+		case 'a':
+			all_set = true;
+			break;
+		case 'd':
+			opt_devid = arg_strtou64(optarg);
+			devid_set = true;
+			break;
+		case 'l':
+			opt_limit = arg_strtou64_with_suffix(optarg);
+			limit_set = true;
+			break;
+		default:
+			usage_unknown_option(cmd, argv);
+		}
+	}
+	if (check_argc_exact(argc - optind, 1))
+		return 1;
+
+	if (devid_set && all_set) {
+		error("--all and --devid cannot be used at the same time");
+		return 1;
+	}
+
+	if (devid_set && !limit_set) {
+		error("--devid and --limit must be set together");
+		return 1;
+	}
+	if (all_set && !limit_set) {
+		error("--all and --limit must be set together");
+		return 1;
+	}
+	if (!all_set && !devid_set && limit_set) {
+		error("--limit must be used with either --all or --devid");
+		return 1;
+	}
+
+	fd = btrfs_open_file_or_dir(argv[optind]);
+	if (fd < 0)
+		return 1;
+
+	ret = ioctl(fd, BTRFS_IOC_FS_INFO, &fi_args);
+	if (ret < 0) {
+		error("failed to read filesystem info: %m");
+		ret = 1;
+		goto out;
+	}
+	if (fi_args.num_devices == 0) {
+		error("no devices found");
+		ret = 1;
+		goto out;
+	}
+	uuid_unparse(fi_args.fsid, fsid);
+	pr_verbose(LOG_DEFAULT, "UUID: %s\n", fsid);
+
+	if (devid_set) {
+		/* Set one device only. */
+		struct btrfs_ioctl_dev_info_args di_args = { 0 };
+		u64 limit;
+
+		ret = device_get_info(fd, opt_devid, &di_args);
+		if (ret == -ENODEV) {
+			error("device with devid %llu not found", opt_devid);
+			ret = 1;
+			goto out;
+		}
+		limit = read_scrub_device_limit(fd, opt_devid);
+		pr_verbose(LOG_DEFAULT, "Set scrub limit of devid %llu from %s%s to %s%s\n",
+			   opt_devid,
+			   limit > 0 ? pretty_size_mode(limit, unit_mode) : "unlimited",
+			   limit > 0 ? "/s" : "",
+			   opt_limit > 0 ? pretty_size_mode(opt_limit, unit_mode) : "unlimited",
+			   opt_limit > 0 ? "/s" : "");
+		ret = write_scrub_device_limit(fd, opt_devid, opt_limit);
+		if (ret < 0) {
+			errno = -ret;
+			error("cannot write to the sysfs file: %m");
+			ret = 1;
+		}
+		ret = 0;
+		goto out;
+	}
+
+	if (all_set && limit_set) {
+		/* Set on all devices. */
+		for (u64 devid = 1; devid <= fi_args.max_id; devid++) {
+			u64 limit;
+			struct btrfs_ioctl_dev_info_args di_args = { 0 };
+
+			ret = device_get_info(fd, devid, &di_args);
+			if (ret == -ENODEV) {
+				continue;
+			} else if (ret < 0) {
+				errno = -ret;
+				error("cannot read devid %llu info: %m", devid);
+				goto out;
+			}
+			limit = read_scrub_device_limit(fd, di_args.devid);
+			pr_verbose(LOG_DEFAULT, "Set scrub limit of devid %llu from %s%s to %s%s\n",
+				   devid,
+				   limit > 0 ? pretty_size_mode(limit, unit_mode) : "unlimited",
+				   limit > 0 ? "/s" : "",
+				   opt_limit > 0 ? pretty_size_mode(opt_limit, unit_mode) : "unlimited",
+				   opt_limit > 0 ? "/s" : "");
+			ret = write_scrub_device_limit(fd, devid, opt_limit);
+			if (ret < 0) {
+				error("cannot write to the sysfs file of devid %llu: %m", devid);
+				goto out;
+			}
+		}
+		ret = 0;
+		goto out;
+	}
+
+	cols = 3;
+	table = table_create(cols, 2 + fi_args.num_devices);
+	if (!table) {
+		error_msg(ERROR_MSG_MEMORY, NULL);
+		ret = 1;
+		goto out;
+	}
+	table->spacing = STRING_TABLE_SPACING_2;
+	idx = 0;
+	table_printf(table, idx++, 0, ">Id");
+	table_printf(table, idx++, 0, ">Limit");
+	table_printf(table, idx++, 0, ">Path");
+	for (int i = 0; i < cols; i++)
+	     table_printf(table, i, 1, "*-");
+
+	for (u64 devid = 1, i = 0; devid <= fi_args.max_id; devid++) {
+		u64 limit;
+		struct btrfs_ioctl_dev_info_args di_args = { 0 };
+
+		ret = device_get_info(fd, devid, &di_args);
+		if (ret == -ENODEV) {
+			continue;
+		} else if (ret < 0) {
+			errno = -ret;
+			error("cannot read devid %llu info: %m", devid);
+			goto out;
+		}
+
+		limit = read_scrub_device_limit(fd, di_args.devid);
+		idx = 0;
+		table_printf(table, idx++, 2 + i, ">%llu", di_args.devid);
+		if (limit > 0) {
+			table_printf(table, idx++, 2 + i, ">%s",
+				     pretty_size_mode(limit, unit_mode));
+		} else {
+			table_printf(table, idx++, 2 + i, ">%s", "-");
+		}
+		table_printf(table, idx++, 2 + i, "<%s", di_args.path);
+		i++;
+	}
+	table_dump(table);
+
+out:
+	if (table)
+		table_free(table);
+	close(fd);
+
+	return !!ret;
+}
+static DEFINE_SIMPLE_COMMAND(scrub_limit, "limit");
+
 static const char scrub_cmd_group_info[] =
 "verify checksums of data and metadata";
 
@@ -1892,6 +2225,7 @@ static const struct cmd_group scrub_cmd_group = {
 		&cmd_struct_scrub_cancel,
 		&cmd_struct_scrub_resume,
 		&cmd_struct_scrub_status,
+		&cmd_struct_scrub_limit,
 		NULL
 	}
 };
