@@ -1228,44 +1228,6 @@ out:
 		btrfs_abort_transaction(trans, ret);
 	return ret;
 }
-static int clear_free_space_tree(struct btrfs_trans_handle *trans,
-				 struct btrfs_root *root)
-{
-	struct btrfs_path *path;
-	struct btrfs_key key;
-	int nr;
-	int ret;
-
-	path = btrfs_alloc_path();
-	if (!path)
-		return -ENOMEM;
-
-	key.objectid = 0;
-	key.type = 0;
-	key.offset = 0;
-
-	while (1) {
-		ret = btrfs_search_slot(trans, root, &key, path, -1, 1);
-		if (ret < 0)
-			goto out;
-
-		nr = btrfs_header_nritems(path->nodes[0]);
-		if (!nr)
-			break;
-
-		path->slots[0] = 0;
-		ret = btrfs_del_items(trans, root, path, 0, nr);
-		if (ret)
-			goto out;
-
-		btrfs_release_path(path);
-	}
-
-	ret = 0;
-out:
-	btrfs_free_path(path);
-	return ret;
-}
 
 int btrfs_clear_free_space_tree(struct btrfs_fs_info *fs_info)
 {
@@ -1288,7 +1250,7 @@ int btrfs_clear_free_space_tree(struct btrfs_fs_info *fs_info)
 
 		while (key.offset < fs_info->nr_global_roots) {
 			free_space_root = btrfs_global_root(fs_info, &key);
-			ret = clear_free_space_tree(trans, free_space_root);
+			ret = btrfs_clear_tree(trans, free_space_root);
 			if (ret)
 				goto abort;
 			key.offset++;
@@ -1299,7 +1261,7 @@ int btrfs_clear_free_space_tree(struct btrfs_fs_info *fs_info)
 			      BTRFS_FEATURE_COMPAT_RO_FREE_SPACE_TREE);
 		btrfs_set_super_compat_ro_flags(fs_info->super_copy, features);
 
-		ret = clear_free_space_tree(trans, free_space_root);
+		ret = btrfs_clear_tree(trans, free_space_root);
 		if (ret)
 			goto abort;
 
