@@ -85,7 +85,7 @@ run_one_test() {
 		fi
 		check_test_results "$RESULTS" "$testname"
 	else
-		_fail "custom test script not found"
+		_fail "custom test script not found or lacks execution permission"
 	fi
 }
 
@@ -95,5 +95,3 @@ for i in $(find "$TEST_TOP/convert-tests" -maxdepth 1 -mindepth 1 -type d \
 do
 	run_one_test "$i"
 done
-
-"$TEST_TOP/scan-results.sh" "$RESULTS"
