@@ -1,12 +1,14 @@
 #!/bin/bash
 # Check if the converted ext2 image is readonly
 
-source "$TOP/tests/common"
-source "$TOP/tests/common.convert"
+source "$TEST_TOP/common" || exit
+source "$TEST_TOP/common.convert" || exit
+
+check_prereq btrfs-convert
+check_global_prereq mke2fs
 
 setup_root_helper
-prepare_test_dev 512M
-check_prereq btrfs-convert
+prepare_test_dev
 
 default_mke2fs="mke2fs -t ext4 -b 4096"
 convert_test_preamble '' 'readonly image test' 16k "$default_mke2fs"
@@ -23,4 +25,4 @@ if [ $? -ne 1 ]; then
 	exit 1
 fi
 run_check_umount_test_dev
-convert_test_post_rollback
+convert_test_post_rollback ext4
