@@ -20,8 +20,7 @@
 #include "libbtrfs/kerncompat.h"
 #include "libbtrfs/version.h"
 #include "libbtrfs/ioctl.h"
-#include "kernel-lib/rbtree.h"
-#include "kernel-lib/list.h"
+#include "kernel-lib/rbtree_types.h"
 #include "kernel-shared/ctree.h"
 #include "kernel-shared/send.h"
 #include "common/send-stream.h"
@@ -33,8 +32,7 @@
  */
 #include "btrfs/kerncompat.h"
 #include "btrfs/version.h"
-#include "btrfs/rbtree.h"
-#include "btrfs/list.h"
+#include "btrfs/rbtree_types.h"
 #include "btrfs/ctree.h"
 #include "btrfs/ioctl.h"
 #include "btrfs/send.h"
@@ -42,6 +40,8 @@
 #include "btrfs/send-utils.h"
 #endif
 
+#include <string.h>
+
 /*
  * Reduced code snippet from snapper.git/snapper/Btrfs.cc
  */
