@@ -2,7 +2,7 @@
 # test that send -p does not corrupt paths when send is using 2 different mount
 # points
 
-source "$TEST_TOP/common"
+source "$TEST_TOP/common" || exit
 
 check_prereq btrfs
 check_prereq mkfs.btrfs
@@ -49,3 +49,6 @@ run_check_umount_test_dev "$TEST_MNT"
 
 run_mayfail $SUDO_HELPER rmdir "$SUBVOL_MNT"
 run_mayfail rmdir "$SUBVOL_MNT"
+
+# don't propagate any potential error from run_mayfail()
+exit 0
